from django.shortcuts import redirect
from django.utils.functional import cached_property
from django.views.generic import FormView, TemplateView, UpdateView

from servala.core.models import Organization
from servala.frontend.forms import OrganizationForm
from servala.frontend.views.mixins import HtmxMixin


class OrganizationCreateView(FormView):
    form_class = OrganizationForm
    template_name = "frontend/organizations/create.html"

    def form_valid(self, form):
        instance = form.instance.create_organization(
            form.instance, owner=self.request.user
        )
        return redirect(instance.urls.base)


class OrganizationDashboardView(TemplateView):
    template_name = "frontend/organizations/dashboard.html"


class OrganizationUpdateView(HtmxMixin, UpdateView):
    template_name = "frontend/organizations/update.html"
    form_class = OrganizationForm
    fragments = ("org-name", "org-name-edit")
    model = Organization
    context_object_name = "organization"

    def get_success_url(self):
        return self.request.path

    def get_object(self):
        return self.request.organization

    @cached_property
    def object(self):
        return self.get_object()

    def form_valid(self, form):
        form.save()
        return super().form_valid(form)
